unit ControlPoints;

interface

uses
  ControlPoint, Graphics;

type
  TControlPoints = class
  public
    cps: array of TControlPoint;
    nrControlPoints: Integer;

    Constructor Create;
    Destructor Destroy; override;

    function LoadFromFile(Filename: string): boolean;

    function GetCP(time: double): TControlPoint;
    procedure AddCP(cp: TControlPoint);
  end;

implementation

uses SysUtils, Classes;

{ TControlPoints }

procedure TControlPoints.AddCP(cp: TControlPoint);
begin
  Inc(nrControlPoints);
  SetLength(cps,nrControlPoints);
  cps[nrControlPoints - 1] := cp;
end;

constructor TControlPoints.Create;
begin
end;

destructor TControlPoints.Destroy;
var
  i: Integer;
begin
  for i := 0 to nrControlPoints - 1 do
    cps[i].Free;

  inherited;
end;


function TControlPoints.GetCP(time: double): TControlPoint;
var
  i: Integer;
begin
  Result := nil;
  if nrControlPoints = 0 then
    Exit;

  if nrControlPoints = 1 then begin
    Result := cps[0].Interpolate(cps[0], cps[0], cps[0].time);
    Exit;
  end;

  if time < cps[0].time then begin
    Result := cps[0].Interpolate(cps[0],cps[0], cps[0].time);
    Exit;
  end;

  if time > cps[nrControlpoints - 1].time then begin
    Result := cps[0].Interpolate(cps[nrControlpoints - 1],cps[nrControlpoints - 1], cps[nrControlpoints - 1].time);
    Exit;
  end;

  for i := 1 to nrControlpoints - 1 do begin
    if cps[i].time > time then begin
      Result := TControlPoint.Interpolate(cps[i - 1], cps[i], time);

      break;
    end;
  end;

end;

function TControlPoints.LoadFromFile(Filename: string): boolean;
var
  filedata: TStringList;
  s: string;
  i: integer;
begin
  Result := False;
  nrControlPoints := 0;

  if not FileExists(Filename) then
    Exit;

  filedata := TStringList.Create;
  filedata.LoadFromFile(Filename);

  s := '';
  for i := 0 to Filedata.Count - 1 do begin
    if trim(Filedata[i]) = ';' then begin
      Inc(nrControlPoints);
      SetLength(cps,nrControlPoints);
      cps[nrControlPoints - 1] := TControlPoint.Create;
      cps[nrControlPoints - 1].ParseString(s);
      s := '';
    end else begin
      s := s + Filedata[i] + ' ';
    end
  end;
  if length(trim(s)) > 0 then begin
    Inc(nrControlPoints);
    SetLength(cps,nrControlPoints);
    cps[nrControlPoints - 1] := TControlPoint.Create;
    cps[nrControlPoints - 1].ParseString(s);
  end;

  filedata.free;

  Result := True;
end;


end.
