unit ifs;

interface

uses
  Classes, Windows, Graphics;

type
  TIFSCode = record
    a, b, c, d, e, f: integer;
    Color: TColor;
    Prob: double;
  end;

type
 PLongintArray = ^TLongintArray;
 TLongintArray = array[0..8192] of Longint;

type
  TIFS = class
  private
    IFSCodes: array of TIFSCode;
    FHeight: integer;
    FWidth: integer;
    faxi, fayi, fbxi, fbyi: Integer;
    Map: Array of Array of integer;
    bm : TBitmap;
    pal: array of integer;
    PropTable: array of integer;
    FNrPixel: Longint;
    FColor: Longint;

    procedure SetHeight(const Value: integer);
    procedure SetWidth(const Value: integer);

    procedure InitMaps;

    procedure ClearMap;
    procedure PreparePropTable;
    procedure PreparePalTable;
    procedure Fillbm;

    procedure Calculate;
    procedure SetColor(const Value: Longint);
    procedure SetNrPixel(const Value: Longint);
  public
    constructor Create;
    destructor  Destroy; override;

    procedure Draw(Canvas: TCanvas; OffsetX: integer = 0; OffsetY: Integer = 0);
    procedure Mutate;
    function  Clone: TIFS;

    Property Width: integer
        read FWidth
       write SetWidth;
    Property Height: integer
        read FHeight
       write SetHeight;
    Property Color: Longint
        read FColor
       write SetColor;
    Property NrPixel: Longint
        read FNrPixel
       write SetNrPixel;

  end;


implementation

uses
  Math,SysUtils;

{ TIFS }

procedure TIFS.Calculate;
var
  i: integer;
  x,y, xr,yr{, xs,ys}: integer;
begin
  ClearMap;

  xr := 0;
  yr := 0;
  for i := 0 to 100 do begin
    with IFSCodes[PropTable[Random(1024)]] do
      try
        x := a * xr + b * yr + e;
        y := c * xr + d * yr + f;
      except
        on EMathError do begin
          raise Exception.Create('Iteration blows up');
          exit;
        end;
      end;
    xr := x div 1024;
    yr := y div 1024;
  end;

  for i := 0 to nrPixel do begin
    with IFSCodes[PropTable[Random(1024)]] do
      try
        x := a * xr + b * yr + e;
        y := c * xr + d * yr + f;
      except
        on EMathError do begin
          raise Exception.Create('Iteration blows up');
          exit;
        end;
      end;
{
    asm
      sar x,10
      sar y,10
    end;
}
    x := x div 1024;
    y := y div 1024;

//    if (abs(x) < 8192) and (y >= -1024) and (y < 11264) then begin
    if (x >= -8192) and (x < 8192) and (y >= -1024) and (y < 11264) then begin
//      xs := (faxi * x + fbxi) div 1048576;
//      ys := (fayi * y + fbyi) div 1048576;
//      Inc(Map[xs,ys]);
      Inc(Map[(faxi * x + fbxi) shr 20, (fayi * y + fbyi) shr 20]);
    end;

    xr := x;
    yr := y;
  end;

  FillBm;
end;

procedure TIFS.ClearMap;
var
  x,y: Integer;
begin
  for y := 0 to FHeight - 1 do
    for x := 0 to FWidth - 1 do
      Map[x,y] := 0;
end;

function TIFS.Clone: TIFS;
begin
  Result := TIFS.Create;
  SetLength(Result.IFSCodes,High(IFSCodes) +1);
  Move(IFSCodes[0], Result.IFSCodes[0], sizeOf(TIFSCode) * (High(IFSCodes) +1));
//  Result.IFSCodes := IFSCodes;
  Result.Height := FHeight;
  Result.Width := FWidth;
  Result.NrPixel := FNrPixel;
  Result.Color := FColor;
end;

constructor TIFS.Create;
begin
  FWidth := 200;
  FHeight := 200;
  Color   := RGB(123,234,12);
  NrPixel := 300000;

  SetLength(IFSCodes, 2);
  IFSCodes[0].a := Round(0.5956 * 1024);
  IFSCodes[0].b := Round(-1.328 * 1024);
  IFSCodes[0].c := Round(0.5382 * 1024);
  IFSCodes[0].d := Round(0.4122 * 1024);
  IFSCodes[0].e := Round(5.951 * 1024 * 1024);
  IFSCodes[0].f := Round(2.504 * 1024 * 1024);
  IFSCodes[0].Prob := 0.9109;

  IFSCodes[1].a := Round(-0.1858 * 1024);
  IFSCodes[1].b := Round(0.1721 * 1024);
  IFSCodes[1].c := Round(0.07634 * 1024);
  IFSCodes[1].d := Round(0.4351 * 1024);
  IFSCodes[1].e := Round(-3.91 * 1024 * 1024);
  IFSCodes[1].f := Round(1.771 * 1024 * 1024);
  IFSCodes[1].Prob := 0.08915;

{
  SetLength(IFSCodes, 3);
  IFSCodes[0].a := Round(0.5 * 1024);
  IFSCodes[0].b := Round(0);
  IFSCodes[0].c := Round(0);
  IFSCodes[0].d := Round(0.5 * 1024);
  IFSCodes[0].e := Round(-4 * 1024 * 1024);
  IFSCodes[0].f := Round(-0.5 * 1024 * 1024);
  IFSCodes[0].Prob := 0.333;
  IFSCodes[1].a := Round(0.5 * 1024);
  IFSCodes[1].b := Round(0);
  IFSCodes[1].c := Round(0);
  IFSCodes[1].d := Round(0.5 * 1024);
  IFSCodes[1].e := Round(4 * 1024 * 1024);
  IFSCodes[1].f := Round(-0.5 * 1024 * 1024);
  IFSCodes[1].Prob := 0.333;
  IFSCodes[2].a := Round(0.5 * 1024);
  IFSCodes[2].b := Round(0);
  IFSCodes[2].c := Round(0);
  IFSCodes[2].d := Round(0.5 * 1024);
  IFSCodes[2].e := Round(0);
  IFSCodes[2].f := Round(5.5 * 1024 * 1024);
  IFSCodes[2].Prob := 0.333;
}

  InitMaps;

  PreparePropTable;
  PreparePalTable;
end;

destructor TIFS.Destroy;
begin
  bm.Free;

  inherited;
end;

procedure TIFS.Draw(Canvas: TCanvas; OffsetX, OffsetY: Integer);
begin
  Calculate;
  Canvas.Draw(OffsetX, OffSetY, bm);
end;

procedure TIFS.Fillbm;
var
  x,y: Integer;
  Row: PLongintArray;
  t: integer;
begin
  for y := 0 to FHeight - 1 do begin
    Row := bm.ScanLine[y];
    for x := 0 to FWidth - 1 do begin
      t := Map[x,y];
      Row[x] := pal[Min(t,255)];
    end;
  end;
end;

procedure TIFS.InitMaps;
begin
  faxi := Round(1024 * FWidth / 16);
  fbxi := Round(1024 * 1024 * FWidth / 2);
  fayi := Round(-1024 * FHeight / 12);
  fbyi := Round(-1024 * 11 * fayi);

  bm := TBitmap.Create;
  bm.PixelFormat := Pf32bit;
  bm.Width := FWidth;
  bm.Height := FHeight;

  setlength(Map, FWidth, FHeight);
end;

procedure TIFS.Mutate;
var
  r,g,b: integer;
  i: integer;
begin
  FNrPixel := Round(FNrPixel * (0.9 + 0.2 * Random));
  r := FColor and $FF;
  g := ((FColor shr 8) and $FF);
  b := ((FColor shr 16) and $FF);
  r := min(255,max(0,round(r - 50 + Random * 100)));
  g := min(255,max(0,round(g - 50 + Random * 100)));
  b := min(255,max(0,round(b - 50 + Random * 100)));
  Color := RGB(r,g,b);

  for i := 0 to High(IFSCodes) do begin
    IFSCodes[i].a := Round(IFSCodes[i].a * (0.9 + 0.2 * Random));
    IFSCodes[i].b := Round(IFSCodes[i].b * (0.9 + 0.2 * Random));
    IFSCodes[i].c := Round(IFSCodes[i].c * (0.9 + 0.2 * Random));
    IFSCodes[i].d := Round(IFSCodes[i].d * (0.9 + 0.2 * Random));
    IFSCodes[i].e := Round(IFSCodes[i].e * (0.9 + 0.2 * Random));
    IFSCodes[i].f := Round(IFSCodes[i].f * (0.9 + 0.2 * Random));
    if random < 0.01 then
      IFSCodes[i].a := -IFSCodes[i].a;
    if random < 0.01 then
      IFSCodes[i].b := -IFSCodes[i].b;
    if random < 0.01 then
      IFSCodes[i].c := -IFSCodes[i].c;
    if random < 0.01 then
      IFSCodes[i].d := -IFSCodes[i].d;
    if random < 0.01 then
      IFSCodes[i].e := -IFSCodes[i].e;
    if random < 0.01 then
      IFSCodes[i].f := -IFSCodes[i].f;
  end;
end;

procedure TIFS.PreparePalTable;
var
  i: Integer;
  r,g,b: integer;
begin
  SetLength(Pal,256);

  pal[0] := 0;
  pal[1] := FColor;
  for i := 2 to 255 do begin
    r := Max(0,(pal[i-1]  and $FF) - 2);
    g := Max(0,((pal[i-1] shr 8) and $FF) - 2);
    b := Max(0,((pal[i-1] shr 16) and $FF) - 2);
    pal[i] := RGB(r,g,b);
  end;
end;

procedure TIFS.PreparePropTable;
var
  i: Integer;
  propsum: double;
  LoopValue: double;
  j: integer;
begin
  SetLength(PropTable, 1024);

  for i := 0 to 1023 do begin
    LoopValue := i/1023;
    propsum := 0;
    j := -1;
    repeat
      inc(j);
      propsum := propsum + IFSCodes[j].Prob;
    until (propsum > LoopValue) or (j = High(IFSCodes));
    PropTable[i] := j;
  end;
end;

procedure TIFS.SetColor(const Value: Longint);
begin
  FColor := Value;
  PreparePalTable;
end;

procedure TIFS.SetHeight(const Value: integer);
begin
  if FHeight <> Value then begin
    FHeight := Value;
    InitMaps;
  end;
end;

procedure TIFS.SetNrPixel(const Value: Longint);
begin
  FNrPixel := Value;
end;

procedure TIFS.SetWidth(const Value: integer);
begin
  if FWidth <> Value then begin
    FWidth := Value;
    InitMaps;
  end;
end;

end.
